/////////////////////////////////////////////////////////////////////////////
// @doc RectObj
// @module RectObj.Java | Implementation of the <c RectObj> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: RectObj.java,v 1.3 2000/08/18 14:23:17 assefa Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: RectObj.java,v $
// Revision 1.3  2000/08/18 14:23:17  assefa
// Use double buffering when drawing (rectangle, circle..) instead of XOR.
//
// Revision 1.2  2000/04/06 11:26:20  gparent
// Fixed a drawing bug using default selection
//
// Revision 1.1  2000/01/28 20:16:30  daniel
// Initial Revision
//
/////////////////////////////////////////////////////////////////////////////

import java.awt.Point;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Color;                      
import java.awt.Dimension;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;                                        
import java.lang.Integer;

/**
 * Used to draw a rectangle. The submit string is a XY coordinate .  Using this object
 * the user could draw an xor rectangle.  When he release the button, the operation
 * is automaticaly commit and the map is notify.
 */

public class RectObj extends DrawObj
  implements MouseMotionListener, MouseListener
{
  /**  This constant is used to know the initial distance before drawing*/
  final protected int knDeltaInit = 2;
  
  /** Used for initial drawing and resizing   */
  protected Point pAnchor;
  /** Used for moving  */
  protected Point pMoveAnchor;
  /** X coordinate at the left top rectangle   */
  protected int x             = 0;
  /** Y coordinate at the left top rectangle   */
  protected int y             = 0;
  /** Width of the current rectangle   */
  protected int width         = 0;
  /** Heigth of the current rectangle  */
  protected int height        = 0;
  /** Use to know if the rectangle is already initialize.  */
  protected boolean bInit     = false;

  /** Parent object.   */
  Map     m_map;

    protected Image       offScreenImage = null;
    protected Graphics 	offScreenGraphics = null;

  /**
   * Constructor.  Add listeners and initialize the members.
   */
  public RectObj( Map parent )
  {
        
    m_map = parent;
    g = parent.getGraphics();

    //Add the events
    addEvent();

  }
  

  /**
   * Create a submit string.  The left top coordinate and right bottom coordinate is
   * send separate by ;
   */
  public String getSubmitValue()
  {
    String strRetVal = Integer.toString( x ) + "," + Integer.toString( y ) + ";" + 
                       Integer.toString( x + width ) + "," + Integer.toString( y + height );
    return strRetVal;
  }

  /**
   * Set the initial point.
   * @param Point in the mouse click.
   */
  public void setAnchor( Point pt )
  {
    pAnchor = pt;
    x           = pAnchor.x;
    y           = pAnchor.y;
    color       = Color.black;
  }

  /**
   * Draw the rectangle.
   */
  public  void draw ()
  {
      if( g == null )      
          g = m_map.getGraphics();    

      if (g != null)
      {
          Color colorCurrent = g.getColor();
          //g.setXORMode(Color.black);
          g.setColor(Color.black);
          g.drawRect( x,  y,  width,  height);
          g.setColor(colorCurrent);
      }

  }

  /**
   * Resize and redraw the new rectangle.
   */
  public void resize (Point p)
  {
    
    Graphics g = m_map.getGraphics();
    if (g != null)
    {
      // debug            
      if( Rosa2000.isDebugMode() == true )
        System.out.println(" Undraw X= " +x +  "y:"  + y +  "width: " + width +  "height: " +  height );

      // Undraw
      //g.setXORMode( Color.black );
      
      computeNewRect(p);

      if( Rosa2000.isDebugMode() == true )
        System.out.println( " draw X= " +x +  "y:"  + y +  "width: " + width +  "height: " +  height );
      
      m_map.redraw();
    }
  }

  /**
   * Draw the initial rectangle.
   */
  public void initPaint ()
  {
    Graphics g = m_map.getGraphics();
    if (g != null)
    {
        //this.setColor(Color.white);
      if( Rosa2000.isDebugMode() == true )
        System.out.println("InitPaint Draw X= " +x +  "y:"  + y +  "width: " + width +  "height: " +  height );

      //g.setXORMode( Color.black );
      m_map.redraw();

      bInit = true;
    }
  }

  /**
   * Compute the new rectangle using the mouse coordinate.
   * @param Mouse location.
   */
  void computeNewRect(Point p)
  {
    if (p.x < pAnchor.x)
    {
      x = p.x;
      width = pAnchor.x - x;
    }
    else
    {
      x = pAnchor.x;
      width = p.x - x;
    }

    if (p.y < pAnchor.y)
    {
      y = p.y;
      height = pAnchor.y - y;
    }
    else
    {
      y = pAnchor.y;
      height = p.y - y;
    }
  }


  //////////////////////////////////////////////////////////////////////////
  // MouseMotionListener implementation

  /**
   * Called when the user drag the rectangle.  For each event, the rectangle will be draw
   * twice.
   */
  public void mouseDragged ( MouseEvent evt )
  {
    if( bInit == false )
    {
      computeNewRect( evt.getPoint() );
      if( width >= knDeltaInit || height >= knDeltaInit )
        initPaint ();
    }
    else
      resize( evt.getPoint());
  }

  /**
   * Destroy the mouse listener and the motion listener.
   */
  public void  destroyEvent()
  {
    m_map.removeMouseListener( this );
    m_map.removeMouseMotionListener( this );
    
  }

    public void  addEvent()
    {
        m_map.addMouseListener( this );
        m_map.addMouseMotionListener( this );
    }
  //////////////////////////////////////////////////////////////////////////
  // MouseListener implementation
  /**
   * Start the drawing rectangle.
   */
  public void mousePressed (MouseEvent evt)
  {
      if( Rosa2000.isDebugMode() == true )
        System.out.println("Anchor  X= " +evt.getX() +  "y:"  + evt.getY()  );

      setAnchor( evt.getPoint() );
  }
  
  /////////////////////////////////
  /**
   * Destroy the event and notify the map.
   */
  public void mouseReleased (MouseEvent evt)
  {
    // Remove the event
      //destroyEvent();

    m_map.commitOperation();
  }
 
    /**
     * reset the object.
     */
    public void reset_object()
    {
        x = 0;
        y = 0;
        width = 0;
        height = 0;
    }
    
  /////////////////////////////////
  // Dummy methods.
  /** Do nothing   */
  public void mouseMoved (MouseEvent evt){}
  /////////////////////////////////
  /** Do nothing   */
  public void mouseEntered (MouseEvent evt){}
  /////////////////////////////////
  /** Do nothing   */
  public void mouseExited (MouseEvent evt){}
  /////////////////////////////////
  /** Do nothing   */
  public void mouseClicked (MouseEvent evt){}
}
