/////////////////////////////////////////////////////////////////////////////
// @doc ToolTipButton
// @module ToolTipButton.Java | Implementation of the <c ToolTipButton> class.
// 
// Rosa2000 Applet<tm>
// <nl>Copyright <cp> 1999, 2000, DM Solutions Group Inc. All rights reserved.
// 
// Author:  Guy Parent
//
// $Id: ToolTipButton.java,v 1.5 2001/05/16 18:37:42 assefa Exp $
// 
/////////////////////////////////////////////////////////////////////////////
// Copyright (c) 1999, 2000, DM Solutions Goup Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the "Software"),
// to deal in the Software without restriction, including without limitation
// the rights to use, copy, modify, merge, publish, distribute, sublicense,
// and/or sell copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
// THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
// DEALINGS IN THE SOFTWARE.
/////////////////////////////////////////////////////////////////////////////
// $Log: ToolTipButton.java,v $
// Revision 1.5  2001/05/16 18:37:42  assefa
// Clean up tooltip threads.
//
// Revision 1.4  2000/08/22 14:14:30  assefa
// Add multi-line support for hints using "|".
//
// Revision 1.3  2000/08/18 20:04:29  assefa
// Check if the hint is set by the user before using it.
//
// Revision 1.2  2000/08/18 14:20:39  assefa
// Add tool tips support.
//
// Revision 1.1  2000/04/18 22:59:29  daniel
// Initial revision (extracted from ToolTipAdapter.java)
//
/////////////////////////////////////////////////////////////////////////////


import java.awt.event.MouseAdapter;
import java.applet.Applet;

import java.awt.Canvas;

import java.awt.Graphics;
import java.awt.Color;                      
import java.awt.Container;                      
import java.awt.Component;

import java.awt.Dimension;
import java.awt.Point;

import java.awt.event.MouseMotionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;                                        
import java.awt.Image;
import java.awt.Toolkit;

import java.awt.Font;
import java.awt.FontMetrics;

import java.util.*;
/*
This is a class that displays the tool tips inside the applet when the
user passes the mouse on top of the buttons (responds to mouse enering 
and mouse leaving events).
To Display the tool tips, It creates an offscreen image of the size of the
tooltip and paints it on top of the applet.
The position of the tooltip is controled by the parent (ButtonCtrl) and 
will depend on the actual position of the button inside the applet.
*/

class ToolTipButton extends Container implements MouseListener
{ 
    protected ButtonCtrl m_parent = null;
    protected Rosa2000 m_applet = null;
    private Point m_Position = new Point(0, 0);
    protected Image       m_offScreenImage = null;
    protected Graphics 	m_offScreenGraphics = null;
    private Dimension m_dimension = new Dimension(-1, -1);
    private Vector m_aStrings = new Vector();
    
    public DelayThread delayThread = new DelayThread();
    
    Font		hintFont;

    public ToolTipButton( Rosa2000 applet, ButtonCtrl parent )
    {
        m_parent = parent;
        m_applet = applet;
       
        hintFont = new Font("Courier", Font.PLAIN, 12);
        delayThread.start();
    }

    /**
     * Return the string displayed as tool tip.
     */
     public String getToolTip(java.awt.event.MouseEvent e) 
    { 
        return m_parent.getHint();
    }
 
    /**
     * Call back when the mouse enters the button area.
     */
    public synchronized void mouseEntered(java.awt.event.MouseEvent e) 
    {
        if (m_parent.getHint() != null)
        {
            determineSize();
            m_Position = m_parent.getHintPosition();
        
            delayThread.arm();
        //paint(m_applet.getGraphics());
        }
    }

    /**
     * Call back when the mouse leaves the button area.
     */
    public synchronized void mouseExited(java.awt.event.MouseEvent e) 
    {   
        if (m_parent.getHint() != null)
        {
            delayThread.disarm();
            hideToolTip();
        //m_applet.repaint(m_Position.x, m_Position.y, getSize().width,
        //                getSize().height);
        }
    }

    /**
     * Call back when the mouse is pressed the button area.
     */
    public synchronized void mousePressed (MouseEvent evt)
    {
        if (m_parent.getHint() != null)
        {
            delayThread.disarm();
            hideToolTip();
        }
    }

    /**
     * Dsiplays the tool tip.
     */
    public void paint(Graphics g)
    {
        if (m_parent.getHint() != null)
        {
            Dimension oSize = this.getSize();
            if (m_offScreenImage == null)
            {
                m_offScreenImage =  m_applet.createImage(oSize.width, oSize.height);
                m_offScreenGraphics = m_offScreenImage.getGraphics();
            }
            m_offScreenGraphics.setColor(new Color(255, 255, 204));
            m_offScreenGraphics.fillRect(0, 0, oSize.width, oSize.height);
     
            m_offScreenGraphics.setColor(new Color(0, 0 , 0));
            m_offScreenGraphics.drawRect(0, 0, oSize.width-1, oSize.height-1);
     
            m_offScreenGraphics.setColor(new Color(0, 0,0));
            m_offScreenGraphics.setFont(hintFont);

            Enumeration enum  = m_aStrings.elements();
            double i = 1.0;
            double dfTotal = (double)m_aStrings.size();
            while (enum.hasMoreElements())
            {
                int nHeight = (int)(((oSize.height) * (i/dfTotal)) -4); 
                m_offScreenGraphics.drawString((String)enum.nextElement(), 2, 
                                               nHeight);
                
                i++;
            }
            //m_offScreenGraphics.drawString(m_parent.getHint(), 2, oSize.height -4);
        
            g.drawImage(m_offScreenImage, m_Position.x, m_Position.y, this);
            //m_applet.showStatus(m_parent.getHint());
        }
    }
    
    /**
     * Utility method to calculate the size of the tool tip using
     * the string and the font.
     */
    private void determineSize()
    {
        Dimension result = null;
        int     nWidth = 0;
        int     nHeight = 0;
        String  sHint = null;
        int     nMaxWidth = -1;

        if (m_dimension.width < 0 || m_dimension.height < 0)
        {
            sHint = m_parent.getHint();
            if ( sHint != null)
            {        
                StringTokenizer st = new StringTokenizer(sHint, "|");
                while (st.hasMoreTokens())
                {
                    String sTmp = st.nextToken();
                    m_aStrings.addElement(new String (sTmp));
                }

                Enumeration enum =  m_aStrings.elements();
                while (enum.hasMoreElements())
                {
                    String sTmp = (String)enum.nextElement();
                    nWidth = 
                        m_applet.getGraphics().getFontMetrics(hintFont).
                        stringWidth(sTmp);
                    if (nWidth > nMaxWidth)
                        nMaxWidth = nWidth;
                }
                
                //nWidth = m_applet.getGraphics().getFontMetrics(hintFont).
                //  stringWidth(m_parent.getHint()) + 4; //4 = 2 space at the beginning && end
                nMaxWidth +=4;
                
                //nHeight = m_applet.getGraphics().getFontMetrics(hintFont).getHeight();
                nHeight =  
                    (m_applet.getGraphics().getFontMetrics(hintFont).
                     getHeight() * m_aStrings.size());// - (m_aStrings.size()*3);
            }
            m_dimension.width = nMaxWidth;
            m_dimension.height = nHeight;

            this.setSize(m_dimension);
        }
    }

    /**
     * Returns the size of the tool tip.
     */
    public Dimension getSize()
    {
        if (m_dimension.width <=0 || m_dimension.height <=0)
          determineSize();
        
        return m_dimension;
    }

    /**
     * Display the tool tip.
     */
    private void showToolTip()
    {
        paint(m_applet.getGraphics());
    }

    /**
     * Hide the tool tip : repaint the applet. 
     */
    private void hideToolTip()
    {
        m_applet.repaint(m_Position.x, m_Position.y, getSize().width,
                         getSize().height);
    }
        
    public void mouseClicked (MouseEvent evt){}
    public void mouseReleased (MouseEvent evt){}
    
    /**
     * Delete tooltip threads.
     */
    public void cleanUp()
    {
        if (delayThread != null)
        {
            //System.out.println("tooltip:cleanUp");
            delayThread.bStopRuning = true;
        }
    }

    /**
     * Utility class to manage the delay used between the moment 
     * the mouse enters the button and the popup of the tooltip.
     * For now the delay is 0.5 second.
     */
    class DelayThread extends Thread
    {
        boolean armed = false;
        boolean reset = false;
        public boolean bStopRuning = false;

        public void run()
        {
            while (true && (bStopRuning == false))
            {
                if (armed)
                {
                    try
                    {
                        sleep(250);
                    }
                    catch (InterruptedException e)
                    {
                    }
                    if (armed && !reset)
                    {
                        armed = false;
                        fire();
                    }

                    reset = false;
                }
                else
                {
                    try
                    {
                        sleep(250);
                    }
                    catch (InterruptedException e)
                    {
                    }
                }
            }
        }

        protected void arm()
        {
            armed = true;
            reset = true;
            interrupt();
        }

        protected void disarm()
        {
            armed = false;
            interrupt();
        }

        protected void reset()
        {
            reset = true;
            interrupt();
        }

        protected void fire()
        {
            showToolTip();
        }
    }
} 
// do not forget to add the listener 
//someTextFieldThatNeedsThePopup.addMouseListener(new SomeToolTip()); 

 
